#include <gkrellm2/gkrellm.h>

#include "libacpi.h"
#include "battery.xpm"

#define	ACPI_CONFIG_KEYWORD	"acpi"

#define PIC_NONE				0
#define PIC_AC					1
#define PIC_BATT				2
#define PIC_BATT_WRN		3
#define PIC_BATT_CH			4
#define PIC_TERM				5
#define PIC_TERM_WRN		6

#define TEMP_UNITS_C		0
#define TEMP_UNITS_K		1
#define TEMP_UNITS_F		2

#define INFO_PERCENTAGE	0
#define INFO_TIME				1

//------------------------------------------------------------------------------

static ACPIInfo acpi_info;
struct acpi_table acpi_t;

static gint panel_batt_on = TRUE, panel_proc_on = TRUE;
static GkrellmMonitor *monitor;
static GkrellmPanel *panel_batt, *panel_proc;
static GkrellmKrell *krell_batt, *krell_proc;
static GkrellmAlert	*acpi_alert_b;
static GkrellmAlert	*acpi_alert_p;
static GkrellmLauncher	launch_batt, launch_proc;
static GkrellmDecal  *decal_ac, *decal_batt, *decal_temp, *decal_term, *decal_throt;

static gint style_id;
static gint info_type = INFO_TIME;
static gint temperature_mode;
static gint last_state = AC_OFF;

//------------------------------------------------------------------------------
/* config tab widgets
*/
static GtkWidget	*enable_acpi_battery_button;
static GtkWidget	*enable_acpi_temperature_button;
static GtkWidget	*batt_launch_entry, *batt_tooltip_entry;
static GtkWidget	*proc_launch_entry, *proc_tooltip_entry;
static GtkWidget	*temperature_mode_button[3];
static GtkWidget	*alert_button;

static gchar			*acpi_info_text[] =
{
N_("Battery:\n"),
N_("\tMiddle click to toggle between percent and remaining time.\n"),
N_("\tWorks only while using battery.\n"),
N_("\nTemperature:\n"),
N_("\tMiddle click to toggle units.\n"),
N_("\t")
};

//------------------------------------------------------------------------------

static gint
panel_batt_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		panel_batt->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		ev->area.width, ev->area.height);
	return FALSE;
}

//------------------------------------------------------------------------------

static gint
panel_proc_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	gdk_draw_pixmap(widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		panel_proc->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
		ev->area.width, ev->area.height);
	return FALSE;
}

//------------------------------------------------------------------------------

static void
update_plugin()
{
  /* remaining time in minutes */
  gchar disp_txt[20];
    
  /* no need to do it every second */
  if (GK.two_second_tick) {
	
		if (panel_batt_on) {
			get_battery_info(&acpi_info, &acpi_t);

			switch (acpi_info.ac) {
				case AC_ON:
					switch (acpi_info.batt) {
						case BATTERY_OK:
							gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_AC);
							break;
						case BATTERY_CRIT:
							gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_BATT_WRN);
							break;
						case BATTERY_CHARGING:
							gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_BATT_CH);
							break;
						default:
							break;
					}
					if (info_type == INFO_TIME)
							info_type = INFO_PERCENTAGE;
					if (last_state == AC_OFF) {
							gkrellm_reset_alert(acpi_alert_b);
							last_state = AC_ON;
					}
					break;
				case AC_OFF:
					switch (acpi_info.batt) {
						case BATTERY:
							gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_BATT);
							break;
						case BATTERY_CRIT:
							gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_BATT_WRN);
							break;
						default:
							break;
					}
					gkrellm_check_alert(acpi_alert_b, acpi_info.batt_percent);
					last_state = AC_OFF;					
					break;
				default:
					break;
			}
			
			if (info_type == INFO_PERCENTAGE) {
				snprintf(disp_txt, 20, " %i%%", (int) (acpi_info.batt_percent));
				gkrellm_draw_decal_text(panel_batt, decal_batt, disp_txt, -1);
			}
			else if (info_type == INFO_TIME) {
				if (acpi_info.batt_minutes < 10)
					snprintf(disp_txt, 20, " %2i:0%1i", acpi_info.batt_hours, acpi_info.batt_minutes);
				else
					snprintf(disp_txt, 20, " %2i:%2i", acpi_info.batt_hours, acpi_info.batt_minutes);
				gkrellm_draw_decal_text(panel_batt, decal_batt, disp_txt, -1);
			}
	
			gkrellm_update_krell(panel_batt, krell_batt, acpi_info.batt_percent);

			gkrellm_draw_panel_layers(panel_batt);
		}
			
		if (panel_proc_on) {
			get_processor_info(&acpi_info, &acpi_t);

			switch (temperature_mode) {
				case TEMP_UNITS_C:
					snprintf(disp_txt, 20, " %iC", (int) (acpi_info.temperature));
					break;
				case TEMP_UNITS_K:
					snprintf(disp_txt, 20, " %iK", (int) (273 + acpi_info.temperature));
					break;
				case TEMP_UNITS_F:
					snprintf(disp_txt, 20, " %iF", (int) (32 + 1.8 * acpi_info.temperature));
					break;
				default:
					break;
			}
			
			gkrellm_check_alert(acpi_alert_p, acpi_info.temperature);
			
			gkrellm_draw_decal_text(panel_proc, decal_temp, disp_txt, -1);
			gkrellm_draw_decal_text(panel_proc, decal_throt, acpi_info.throt_state, -1);
			
			gkrellm_update_krell(panel_proc, krell_proc, 100. * acpi_info.temperature / acpi_info.temperature_crit);
	 
			gkrellm_draw_panel_layers(panel_proc);
		}
  }
}

//------------------------------------------------------------------------------

static gint
panel_batt_press(GtkWidget *widget, GdkEventButton *ev)	{
	if (ev->button == 3)
		gkrellm_open_config_window(monitor);
	
	else if (ev->button == 2) {
		switch (info_type) {
			case INFO_PERCENTAGE:
				info_type = INFO_TIME;
				update_plugin();
				break;
			case INFO_TIME:
				info_type = INFO_PERCENTAGE;
				update_plugin();
				break;
			default:
				break;
		}
	}
	return TRUE;
}
	
//------------------------------------------------------------------------------

static gint
panel_proc_press(GtkWidget *widget, GdkEventButton *ev)	{
	if (ev->button == 3)
		gkrellm_open_config_window(monitor);
	
	else if (ev->button == 2) {
		switch (temperature_mode) {
			case TEMP_UNITS_C:
				temperature_mode = TEMP_UNITS_K;
				break;
			case TEMP_UNITS_K:
				temperature_mode = TEMP_UNITS_F;
				break;
			case TEMP_UNITS_F:
				temperature_mode = TEMP_UNITS_C;
				break;
			default:
				break;
		}
	}
	return TRUE;
}

//------------------------------------------------------------------------------

static void
create_plugin(GtkWidget *vbox, gint first_create)
{
  GkrellmStyle *style;
  GkrellmTextstyle *ts;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GkrellmPiximage *krell_image;
    
	if (first_create) {
		panel_batt = gkrellm_panel_new0();
		panel_proc = gkrellm_panel_new0();
	}

	style = gkrellm_meter_style(style_id);
	
	/* battery krell
	*/
	
	krell_image = gkrellm_krell_meter_piximage(style_id);
	krell_batt = gkrellm_create_krell(panel_batt, krell_image, style);
	gkrellm_monotonic_krell_values(krell_batt, FALSE);
	gkrellm_set_krell_full_scale(krell_batt, 100, 1);

	krell_proc = gkrellm_create_krell(panel_proc, krell_image, style);
	gkrellm_monotonic_krell_values(krell_proc, FALSE);
	gkrellm_set_krell_full_scale(krell_proc, 100, 1);

	/* temperature text
	*/
	ts = gkrellm_meter_textstyle(style_id);

	pixmap = gdk_pixmap_create_from_xpm_d (vbox->window, &mask, NULL, (char **) battery_xpm);

	decal_batt = gkrellm_create_decal_text(panel_batt, "XXXC", ts, style, (gkrellm_get_style_margins(style))->left, 12, -1);

	decal_ac = gkrellm_create_decal_pixmap(panel_batt, pixmap, mask, 7, NULL, - 20, 12);

  decal_temp = gkrellm_create_decal_text(panel_proc, "XXXC", ts, style, (gkrellm_get_style_margins(style))->left, 12, -1);
  decal_throt = gkrellm_create_decal_text(panel_proc, "--", ts, style, (gkrellm_get_style_margins(style))->left, 12, -1);

	decal_term = gkrellm_create_decal_pixmap(panel_proc, pixmap, mask, 7, NULL, - 20, 12);

/* Configure the panel to hold the above created decals, and create it.
	*/
  gkrellm_panel_configure(panel_batt, NULL, style);
  gkrellm_panel_configure(panel_proc, NULL, style);

	/*  create panel
	*/
  gkrellm_panel_create(vbox, monitor, panel_batt);
  gkrellm_panel_create(vbox, monitor, panel_proc);
    
	gkrellm_move_decal(panel_batt, decal_ac, panel_batt->w - decal_ac->w - (gkrellm_get_style_margins(style))->right - 2, 12);

	gkrellm_draw_decal_text(panel_proc, decal_throt, "--", 1);
	gkrellm_move_decal(panel_proc, decal_throt, panel_proc->w - decal_term->w - decal_throt->w - (gkrellm_get_style_margins(style))->right - 6, 12);
	gkrellm_move_decal(panel_proc, decal_term, panel_proc->w - decal_term->w - (gkrellm_get_style_margins(style))->right - 2, 12);
	  
	gkrellm_draw_decal_text(panel_batt, decal_batt, "----", 1);
	gkrellm_draw_decal_pixmap(panel_batt, decal_ac, PIC_AC);

	gkrellm_draw_decal_text(panel_proc, decal_temp, "XXXC", 1);
	gkrellm_draw_decal_text(panel_proc, decal_throt, "--", 1);
	gkrellm_draw_decal_pixmap(panel_proc, decal_term, PIC_TERM);

	/* first create
	*/
	if (first_create) {
    gtk_signal_connect(GTK_OBJECT (panel_batt->drawing_area), "expose_event", (GtkSignalFunc) panel_batt_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (panel_batt->drawing_area), "button_press_event", (GtkSignalFunc) panel_batt_press, NULL);

		gtk_signal_connect(GTK_OBJECT (panel_proc->drawing_area), "expose_event", (GtkSignalFunc) panel_proc_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT (panel_proc->drawing_area), "button_press_event", (GtkSignalFunc) panel_proc_press, NULL);
	}

	gkrellm_setup_launcher(panel_batt, &launch_batt, METER_PANEL_TYPE, 0);
	gkrellm_setup_launcher(panel_proc, &launch_proc, METER_PANEL_TYPE, 0);
	
	if (!panel_batt_on)
		gkrellm_panel_hide(panel_batt);
	if (!panel_proc_on)
		gkrellm_panel_hide(panel_proc);

}

//------------------------------------------------------------------------------

/* plugin config
*/	
static void
acpi_alert_b_trigger(GkrellmAlert *alert, gpointer data)	{
	GkrellmAlertdecal	*ad;

	alert->panel = panel_batt;
	ad = &alert->ad;
	gkrellm_render_default_alert_decal(alert);
}

static void
acpi_alert_p_trigger(GkrellmAlert *alert, gpointer data)	{
	GkrellmAlertdecal	*ad;

	alert->panel = panel_proc;
	ad = &alert->ad;
	gkrellm_render_default_alert_decal(alert);
}
//------------------------------------------------------------------------------

static void
create_acpi_alert_b(void) {
	if (!acpi_alert_b)
		acpi_alert_b = gkrellm_alert_create(NULL, _("ACPI"), _("Battery Remaining Limits"), 
																				FALSE, TRUE, TRUE, 
																				500, 0,
																				1, 10, 0);
	
	gkrellm_alert_trigger_connect(acpi_alert_b, acpi_alert_b_trigger, NULL);
}
	
static void
create_acpi_alert_p(void) {
	if (!acpi_alert_p)
		acpi_alert_p = gkrellm_alert_create(NULL, _("ACPI"), _("Temperature Limits"), 
																				FALSE, FALSE, TRUE, 
																				500, 0, 
																				1, 10, 0);
	
	gkrellm_alert_trigger_connect(acpi_alert_p, acpi_alert_p_trigger, NULL);
}

//------------------------------------------------------------------------------

static void
acpi_set_alert_b(GtkWidget *entry, gpointer data)
	{
	create_acpi_alert_b();
	gkrellm_alert_config_window(&acpi_alert_b);
}

static void
acpi_set_alert_p(GtkWidget *entry, gpointer data)
	{
	create_acpi_alert_p();
	gkrellm_alert_config_window(&acpi_alert_p);
}

//------------------------------------------------------------------------------

static void
save_acpi_config(FILE *f) {
	fprintf(f, "%s enable_batt %d\n", ACPI_CONFIG_KEYWORD, panel_batt_on);
	fprintf(f, "%s enable_proc %d\n", ACPI_CONFIG_KEYWORD, panel_proc_on);
	fprintf(f, "%s launch_batt %s\n", ACPI_CONFIG_KEYWORD, launch_batt.command);
	fprintf(f, "%s batt_tooltip_comment %s\n",	ACPI_CONFIG_KEYWORD, launch_batt.tooltip_comment);
	fprintf(f, "%s launch_proc %s\n", ACPI_CONFIG_KEYWORD, launch_proc.command);
	fprintf(f, "%s proc_tooltip_comment %s\n",	ACPI_CONFIG_KEYWORD, launch_proc.tooltip_comment);
	fprintf(f, "%s proc_mode %d\n",	ACPI_CONFIG_KEYWORD, temperature_mode);
	
	gkrellm_save_alertconfig(f, acpi_alert_b, ACPI_CONFIG_KEYWORD, "batt");

	gkrellm_save_alertconfig(f, acpi_alert_p, ACPI_CONFIG_KEYWORD, "proc");	
}

//------------------------------------------------------------------------------

static void
load_acpi_config(gchar *arg)	{
	gchar	config[32], item[CFG_BUFSIZE];
	gchar s[32];
	
	if (sscanf(arg, "%31s %[^\n]", config, item) == 2) {
		if (!strcmp(config, "enable_batt"))
			sscanf(item, "%d", &panel_batt_on);
		if (!strcmp(config, "enable_proc"))
			sscanf(item, "%d", &panel_proc_on);
		if (!strcmp(config, "proc_mode"))
			sscanf(item, "%d", &temperature_mode);
		if (!strcmp(config, "launch_batt"))
			launch_batt.command = g_strdup(item);
		if (!strcmp(config, "batt_tooltip_comment"))
			launch_batt.tooltip_comment = g_strdup(item);
		if (!strcmp(config, "launch_proc"))
			launch_proc.command = g_strdup(item);
		if (!strcmp(config, "proc_tooltip_comment"))
			launch_proc.tooltip_comment = g_strdup(item);
		
		if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD)) {
			sscanf(item, "%s", s);
			fprintf(stderr, "%s %s\n", s, item);
			
			if (!strcmp(s, "batt")) {
				create_acpi_alert_b();
				gkrellm_load_alertconfig(&acpi_alert_b, item);
			}
			else if (!strcmp(s, "proc")) {
				create_acpi_alert_p();
				gkrellm_load_alertconfig(&acpi_alert_p, item);
			}
		}
	}
}	

//------------------------------------------------------------------------------

static void 
apply_acpi_config(void) {
	gint	new, i;

	new = GTK_TOGGLE_BUTTON(enable_acpi_battery_button)->active;
	panel_batt_on = new;
	
	if (panel_batt_on)
		gkrellm_panel_show(panel_batt);
	else
		gkrellm_panel_hide(panel_batt);
	
	new = GTK_TOGGLE_BUTTON(enable_acpi_temperature_button)->active;
	panel_proc_on = new;
	
	if (panel_proc_on)
		gkrellm_panel_show(panel_proc);
	else
		gkrellm_panel_hide(panel_proc);
	
	for (i = 0; i < 3; ++i)
		if (GTK_TOGGLE_BUTTON(temperature_mode_button[i])->active)
			temperature_mode = i;

	gkrellm_apply_launcher(&batt_launch_entry, &batt_tooltip_entry, panel_batt, &launch_batt, gkrellm_launch_button_cb);
	gkrellm_apply_launcher(&proc_launch_entry, &proc_tooltip_entry, panel_proc, &launch_proc, gkrellm_launch_button_cb);
}
	
//------------------------------------------------------------------------------

static void
create_acpi_tab(GtkWidget *tab_vbox) {
	GtkWidget	*tabs, *table, *vbox, *vbox1, *hbox, *text, *button, *label1;
	GSList *group = NULL;
	gint		i;

	tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

/* -- Battery tab */
	vbox = gkrellm_gtk_notebook_page(tabs, _("Battery"));

	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Options"),	4, FALSE, 0, 2);
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gkrellm_gtk_check_button(hbox, &enable_acpi_battery_button, panel_batt_on, FALSE, 10, _("Enable ACPI battery"));
	gtk_box_pack_start(GTK_BOX(hbox), enable_acpi_battery_button, FALSE, FALSE, 0);
	gkrellm_gtk_button_connected(hbox, &alert_button, TRUE, FALSE, 0, acpi_set_alert_b, NULL, _("Alerts"));
	gtk_box_pack_end(GTK_BOX(hbox), alert_button, FALSE, FALSE, 0);		

	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),	4, FALSE, 0, 2);
	table = gkrellm_gtk_launcher_table_new(vbox1, 1);
	gkrellm_gtk_config_launcher(table, 0, &batt_launch_entry, &batt_tooltip_entry, _("Battery"), &launch_batt);

/* -- Temperature tab */
	vbox = gkrellm_gtk_notebook_page(tabs, _("Temperature"));

	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Options"),	4, FALSE, 0, 2);
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
	gkrellm_gtk_check_button(hbox, &enable_acpi_temperature_button, panel_proc_on, FALSE, 10, _("Enable ACPI temperature"));
	gtk_box_pack_start(GTK_BOX(hbox), enable_acpi_temperature_button, FALSE, FALSE, 0);
	gkrellm_gtk_button_connected(hbox, &alert_button, TRUE, FALSE, 0, acpi_set_alert_p, NULL, _("Alerts"));
	gtk_box_pack_end(GTK_BOX(hbox), alert_button, FALSE, FALSE, 0);		
	
	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Temperature units"),	4, FALSE, 0, 2);
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);

	button = gtk_radio_button_new_with_label(group, _("Celsius"));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
	temperature_mode_button[0] = button;
	group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));

	button = gtk_radio_button_new_with_label(group, _("Kelvin"));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
	temperature_mode_button[1] = button;
	group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
	
	button = gtk_radio_button_new_with_label(group, _("Fahrenheit"));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
	temperature_mode_button[2] = button;
	group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));

	button = temperature_mode_button[temperature_mode];
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),	4, FALSE, 0, 2);
	table = gkrellm_gtk_launcher_table_new(vbox1, 1);
	gkrellm_gtk_config_launcher(table, 0, &proc_launch_entry, &proc_tooltip_entry, _("Temperature"), &launch_proc);

/* --About tab */
	vbox = gkrellm_gtk_notebook_page(tabs, _("Info"));
	text = gkrellm_gtk_scrolled_text_view(vbox, NULL, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	for (i = 0; i < sizeof(acpi_info_text)/sizeof(gchar *); ++i)
		gkrellm_gtk_text_view_append(text, _(acpi_info_text[i]));

/* --About tab */
	vbox = gkrellm_gtk_notebook_page(tabs, _("About"));
	label1 = gtk_label_new (_("GKACPI\nGKrellM ACPI plugin\n\ncopyright (c) 2002 Michal Suski\nsuseu@poczta.onet.pl\nhttp://gkacpi.sourceforge.net\n\nreleased under GNU Public License"));
  gtk_box_pack_start (GTK_BOX (vbox), label1, TRUE, TRUE, 0);
	}

//------------------------------------------------------------------------------
/* The monitor structure tells GKrellM how to call the plugin routines.
*/
static GkrellmMonitor	plugin_mon	=
	{
	"GKACPI",								// Name, for config tab
	0,											// Id,  0 if a plugin
	create_plugin,					// The create function
	update_plugin,					// The update function
	create_acpi_tab,				// The config tab create function
	apply_acpi_config,			// Apply the config function
		
	save_acpi_config,				// Save user config
	load_acpi_config,				// Load user config
	ACPI_CONFIG_KEYWORD,		// config keyword

	NULL,										// Undefined 2
	NULL,										// Undefined 1
	NULL,										// private

	MON_APM,								// Insert plugin before this monitor

	NULL,										// Handle if a plugin, filled in by GKrellM
	NULL										// path if a plugin, filled in by GKrellM
	};

//------------------------------------------------------------------------------
/* All GKrellM plugins must have one global routine named init_plugin()
which returns a pointer to a filled in monitor structure.
*/
GkrellmMonitor *
gkrellm_init_plugin()
{
  /* before creating the plugin we initialise the ACPI check
  */
  if (initialise_acpi(&acpi_info, &acpi_t)) {
		monitor = &plugin_mon;
		return &plugin_mon;
	}
	else {
			gkrellm_config_message_window(_("ACPI Config Error"),
				_("No battery available."), NULL);
		return NULL;
	}
}
