#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>

#include "libacpi.h"

static ACPIFiles acpi_files;

static void get_design_cap();

static int full_cap;
static int low_cap;

//------------------------------------------------------------------------------

/* check if acpi is present and sane	
*/
int
initialise_acpi(ACPIInfo *a, struct acpi_table *a_t) {
  FILE *f;
  char s[128];
    
  f = fopen("/proc/acpi/info", "r");
  if (!f) {
    fprintf(stderr, "This system does not support ACPI.\n");
  
    return 0;
  }
  else {
    while (fgets(s, 128, f) != NULL) {
      char s1[25], s2[25];

      /* read info
	  */
	  sscanf(s, "%s %s", s1, s2);

      if (strcmp(s1, "version:") == 0)
		sprintf(a->ver, "%s\n", s2);
    }
    fclose(f);
  }
	
//  if (!set_files()) {
//		fprintf(stderr, "This system supports old ACPI. Patch your kernel.\n");
	  
//		return 0;		
//  }
  
  acpi_set_device_table(a_t);
  acpi_print_device_table(a_t);
		
  get_design_cap(a, a_t);

  return 1;
}   

//------------------------------------------------------------------------------
/* acpi find specific device
*/
struct acpi_device *
acpi_new_device			(char *d_d) {
     struct acpi_device *tmp;
     
     tmp = malloc(sizeof(struct acpi_device));
     
     tmp->device_name = malloc(strlen(d_d));
     
     strcpy(tmp->device_name, d_d);
		 
     tmp->next = NULL;
     
     return tmp;
}

//------------------------------------------------------------------------------
/* acpi find specific device
*/
struct acpi_device * 
acpi_find_device		(char *d_n) {
    DIR *d;
    struct dirent *de;
    struct acpi_device *device = NULL;
    struct acpi_device *device_tmp = NULL;
    int i = 0;

    if (chdir(ACPI_PATH) < 0) {
        fprintf(stderr, "No ACPI support in kernel.\n");
        exit(1);
    }
    
    if (chdir(d_n) < 0) {
        fprintf(stderr, "No support for device type: %s.\n", d_n);
        return NULL;
    }
    
    d = opendir(".");
    if (!d) {
        return NULL;
    }
    
    while ((de = readdir(d))) {
        if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, "..")) 
            continue;
        
	if (!device) {
	    device = acpi_new_device(de->d_name);
	    device_tmp = device;
	    i++;
	}
	else {
	    device_tmp->next = acpi_new_device(de->d_name);
	    device_tmp = device_tmp->next;
	    i++;
	}
    }
    closedir(d);
    
    return device;
}

//------------------------------------------------------------------------------
/* acpi device table
*/
int
acpi_set_device_table		(struct acpi_table *a_t) {

    a_t->dev_ac = acpi_find_device(AC_DEV_NAME);
    a_t->dev_batt = acpi_find_device(BATT_DEV_NAME);
    a_t->dev_butt = acpi_find_device(BUTT_DEV_NAME);
    a_t->dev_embd_cntr = acpi_find_device(EMBD_CNTR_DEV_NAME);
    a_t->dev_fan = acpi_find_device(FAN_DEV_NAME);
    a_t->dev_pwr_rsrc = acpi_find_device(PWR_RSRC_DEV_NAME);
    a_t->dev_proc = acpi_find_device(PROC_DEV_NAME);
    a_t->dev_thrm = acpi_find_device(THRM_DEV_NAME);

    return 0;
}

//------------------------------------------------------------------------------
/* acpi print device table
*/
void
acpi_print_device		(struct acpi_device *a_d) {
    struct acpi_device *dev;
    int i = 0;
    
    dev = a_d;
    
    while (dev) {
			fprintf(stderr, "%d %s\n", i, dev->device_name);
			dev = dev->next;
			i++;
    }
}

//------------------------------------------------------------------------------
/* acpi print device table
*/
void
acpi_print_device_table		(struct acpi_table *a_t) {

    fprintf(stderr, "devices:\n");
    
    acpi_print_device(a_t->dev_ac);
    acpi_print_device(a_t->dev_batt);
    acpi_print_device(a_t->dev_butt);
    acpi_print_device(a_t->dev_embd_cntr);
    acpi_print_device(a_t->dev_fan);
    acpi_print_device(a_t->dev_pwr_rsrc);
    acpi_print_device(a_t->dev_proc);
    acpi_print_device(a_t->dev_thrm);
}

//------------------------------------------------------------------------------
/* acpi file names
*/
int 
set_files() {
  FILE *f;
    
  f = fopen("/proc/acpi/battery/BAT0/info", "r");
  if (!f) {
	f = fopen("/proc/acpi/battery/BAT1/info", "r");
	if (!f) {
	  fprintf(stderr, "Can't find any battery.\n");
			
	  return 0;
	}

	sprintf(acpi_files.batt_info, "/proc/acpi/battery/BAT1/info");
	sprintf(acpi_files.batt_state, "/proc/acpi/battery/BAT1/state");
  }
  else {
	sprintf(acpi_files.batt_info, "/proc/acpi/battery/BAT0/info");
	sprintf(acpi_files.batt_state, "/proc/acpi/battery/BAT0/state");
  }
	
  f = fopen("/proc/acpi/ac_adapter/AC/state", "r");
  if (!f) {
	fprintf(stderr, "Can't find AC adapter.\n");
			
	return 0;
  }
  else {
	sprintf(acpi_files.ac_state, "/proc/acpi/ac_adapter/AC/state");		
  }
	
  f = fopen("/proc/acpi/thermal_zone/THRM/temperature", "r");
  if (!f) {
	fprintf(stderr, "Can't find thermal zone.\n");
			
	return 0;
  }
  else {
	sprintf(acpi_files.thrm_temp, "/proc/acpi/thermal_zone/THRM/temperature");		
  }
	
  f = fopen("/proc/acpi/thermal_zone/THRM/trip_points", "r");
  if (!f) {
	fprintf(stderr, "Can't find thermal zone.\n");
			
	return 0;
  }
  else {
	sprintf(acpi_files.thrm_crit, "/proc/acpi/thermal_zone/THRM/trip_points");		
  }

  f = fopen("/proc/acpi/processor/CPU0/throttling", "r");
  if (!f) {
	fprintf(stderr, "Can't find CPU throttling.\n");
			
	return 0;
  }
  else {
	sprintf(acpi_files.cpu_throt, "/proc/acpi/processor/CPU0/throttling");		
  }

  return 1;
}

//------------------------------------------------------------------------------

/* battery info
*/
void
get_battery_info(ACPIInfo *a, struct acpi_table *a_t)
{
  FILE *f;
	char f_name[128];
  char s[128];
  int got_info = 0;

  int cur_rate = 0;
	int cur_hours = 0;
	int cur_minutes = 0;
  int cur_cap = 0;
  int cur_percent = 0;
    
	sprintf(f_name, "%s/%s/%s/state", ACPI_PATH, AC_DEV_NAME, a_t->dev_ac->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while (fgets(s, 128, f) != NULL) {
      char s1[25], s2[25];

      /* read ac status
			*/
			sscanf(s, "%s %s", s1, s2);

      if (strcmp(s1, "state:") == 0) {
        if (strcmp(s2, "on-line") == 0)
          a->ac = AC_ON;
        if (strcmp(s2, "off-line") == 0)
          a->ac = AC_OFF;
      }
    }
    fclose(f);
  }
	else {
		printf("Failed to open %s\n", f_name);
	}
    
  got_info = 0;
	
	sprintf(f_name, "%s/%s/%s/state", ACPI_PATH, BATT_DEV_NAME, a_t->dev_batt->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while ((fgets(s, 128, f) != NULL) && (got_info != 3)) {
      char s1[25], s2[25], s3[25];

			/*read batery status
			*/
      sscanf(s, "%s %s %s", s1, s2, s3);

      if (strcmp(s1, "charging") == 0) {
        if (strcmp(s3, "unknown") == 0)
          a->batt = BATTERY_OK;
        if (strcmp(s3, "charging") == 0)
          a->batt = BATTERY_CHARGING;
        if (strcmp(s3, "discharging") == 0)
          a->batt = BATTERY;

        got_info++;
			}

			if (strcmp(s1, "remaining") == 0) {
        cur_cap = atoi(s3);
        got_info++;
      }

 			if (strcmp(s2, "rate:") == 0) {
        cur_rate = atoi(s3);
				
				if ((a->batt ==BATTERY_CHARGING) && (cur_rate == 0))
					a->batt = BATTERY_OK;
					
        got_info++;
      }

		}
		fclose(f);
  }
	else {
		printf("Failed to open %s\n", f_name);
	}
	
	got_info = 0;

  /* calculations
	*/
	cur_percent = ((float) cur_cap / (float) full_cap) * 100;
	if (cur_percent > 100)
			cur_percent = 100;
	
	if (cur_rate >0) {
		cur_hours = cur_cap / cur_rate;
		cur_minutes = 60 * (((float) cur_cap / (float) cur_rate) - (float) cur_hours);
	}
	 
	if (cur_cap <= low_cap)
    a->batt = BATTERY_CRIT;

  /* store information
	*/
	a->batt_percent = cur_percent;
	a->batt_hours = cur_hours;
	a->batt_minutes = cur_minutes;
}

//------------------------------------------------------------------------------

/* throttling info
*/
void
get_processor_info(ACPIInfo *a, struct acpi_table *a_t)
{
  FILE *f;
  char s[128];
	char f_name[128];
  int got_info = 0;
	
	sprintf(f_name, "%s/%s/%s/temperature", ACPI_PATH, THRM_DEV_NAME, a_t->dev_thrm->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while (fgets(s, 128, f) != NULL) {
      char s1[25], s2[25], s3[25];

			/* read temperature
			*/
			sscanf(s, "%s %s %s", s1, s2, s3);

      if (strcmp(s1, "temperature:") == 0) {
        a->temperature = atoi(s2);
        got_info++;
      }
		}
		fclose(f);
  }
	else {
		printf("Failed to open %s\n", f_name);
	}

  got_info = 0;

	sprintf(f_name, "%s/%s/%s/throttling", ACPI_PATH, PROC_DEV_NAME, a_t->dev_proc->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while (fgets(s, 128, f) != NULL) {
      char s1[25], s2[25], s3[25];

      /* read ac status
			*/
			sscanf(s, "%s %s %s", s1, s2, s3);

      if (strcmp(s1, "active") == 0) {
				sprintf(a->throt_state, "%s", s3);
      }
    }
    fclose(f);
  }
	else {
		printf("Failed to open %s\n", f_name);
	}
}

//------------------------------------------------------------------------------

/* battery pre-info
*/
void
get_design_cap(ACPIInfo *a, struct acpi_table *a_t)
{
  FILE *f;
  char s[128];
	char f_name[128];
  int got_info = 0;
    
	sprintf(f_name, "%s/%s/%s/info", ACPI_PATH, BATT_DEV_NAME, a_t->dev_batt->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while ((fgets(s, 128, f) != NULL) && got_info != 2) {
      char s1[25], s2[25], s3[25], s4[25];

      sscanf(s, "%s %s %s %s", s1, s2, s3, s4);
                
      if (strcmp(s1, "last") == 0) {
        full_cap = atoi(s4);
        got_info++;
      }

      if (strcmp(s3, "warning:") == 0) {
        low_cap = atoi(s4);
        got_info++;
      }
    }
    fclose(f);
  }
	else {
		printf("Failed to open %sn", f_name);
	}

	sprintf(f_name, "%s/%s/%s/trip_points", ACPI_PATH, THRM_DEV_NAME, a_t->dev_thrm->device_name);
  f = fopen(f_name, "r");
  if (f) {
    while (fgets(s, 128, f) != NULL) {
      char s1[25], s2[25], s3[25];

      sscanf(s, "%s %s %s", s1, s2, s3);

      if (strcmp(s1, "critical") == 0) {
        a->temperature_crit = atoi(s3);
        got_info++;
      }
		}
		fclose(f);
  }
	else {
		printf("Failed to open %s\n", f_name);
	}
}
