#ifndef _LIBACPI_H_
#define _LIBACPI_H_

#define ACPI_PATH			"/proc/acpi"

#define AC_DEV_NAME			"ac_adapter"
#define BATT_DEV_NAME		"battery"
#define BUTT_DEV_NAME		"button"
#define EMBD_CNTR_DEV_NAME	"embedded_controller"
#define FAN_DEV_NAME		"fan"
#define PWR_RSRC_DEV_NAME	"power_resource"
#define PROC_DEV_NAME		"processor"
#define THRM_DEV_NAME		"thermal_zone"

#define AC_ON				0
#define AC_OFF				1

#define BATTERY_OK			0
#define BATTERY				1
#define BATTERY_CRIT		2
#define BATTERY_CHARGING	3

//------------------------------------------------------------------------------

struct acpi_device {
  char *device_name;
  void *device_data;
  struct acpi_device *next;
};

//------------------------------------------------------------------------------

struct acpi_table {
  struct acpi_device *dev_ac;
  struct acpi_device *dev_batt;
  struct acpi_device *dev_butt;
  struct acpi_device *dev_embd_cntr;
  struct acpi_device *dev_fan;
  struct acpi_device *dev_pwr_rsrc;
  struct acpi_device *dev_proc;
  struct acpi_device *dev_thrm;
};

//------------------------------------------------------------------------------

struct acpi_info {
  struct acpi_info_ac *ac;
};

//------------------------------------------------------------------------------

typedef struct {
	char ver[10];
	
	int ac;
	int batt;
	
	int batt_capacity;
	int batt_percent;
	int batt_hours;
	int batt_minutes;
	int temperature;
	int temperature_crit;
	int throt;
	int throt_limit;
	
	char throt_state[5];
} ACPIInfo;

//------------------------------------------------------------------------------

typedef struct {
	char batt_info[50];
	char batt_state[50];
	char ac_state[50];
	char thrm_temp[50];
	char thrm_crit[50];
	char cpu_throt[50];
	
} ACPIFiles;

//------------------------------------------------------------------------------

int initialise_acpi(ACPIInfo *a, struct acpi_table *a_t);
int set_files();

struct acpi_device *
acpi_new_device				(char *d_d);

struct acpi_device *
acpi_find_device			(char *d_n);
						
int	
acpi_set_device_table		(struct acpi_table *a_t);

void 
acpi_print_device			(struct acpi_device *a_d);

void 
acpi_print_device_table		(struct acpi_table *a_t);

void get_battery_info(ACPIInfo *, struct acpi_table *);
void get_processor_info(ACPIInfo *, struct acpi_table *);

#endif
